package it.unitn.web.protocol;

import java.net.*;
import java.io.*;
import java.util.*;

/**
  * Simple protocol implementation. Allows obtaining information 
  * on the indicated file and file retrieval to the authorized users.
  *
  * @version 1.6 07/03/06
  * @author Alexander Ivanyukovich
  */
public class TestProtocol {
  private static final byte NOTAUTHORISED = 0; //->AUTHORISED (after successful LOGIN)
                                               //->NOTAUTHORISED (otherwise)
  private static final byte AUTHORISED = 1; //->NOTAUTHORISED (after LOGOUT)
                                            //->AUTHORISED (otherwise)

  private static final String STATUS_OK = "OK";
  private static final String STATUS_FAILED = "FAILED";
  private static final String STATUS_COMMAND_UNKNOWN = "COMMAND_UNKNOWN";

  //commands
  private static final byte COMMAND_EXIT      = 0;
  private static final byte COMMAND_LOGIN     = 1;
  private static final byte COMMAND_LOGOUT    = 2;
  private static final byte COMMAND_FILEINFO  = 3;
  private static final byte COMMAND_FILEGET   = 4;
  private static final byte COMMAND_STATUS    = 5;
  private static final byte COMMAND_HELP      = 6;
  
  private static Hashtable<String,Byte> commands = new Hashtable<String,Byte>();
  
  static{
    commands.put("exit",     COMMAND_EXIT);
    commands.put("login",    COMMAND_LOGIN);
    commands.put("logout",   COMMAND_LOGOUT);
    commands.put("fileinfo", COMMAND_FILEINFO);
    commands.put("fileget",  COMMAND_FILEGET);
    commands.put("status",   COMMAND_STATUS);
    commands.put("help",     COMMAND_HELP);
  }

  private static final String LOGIN = "admin";
  private static final String PASSWORD = "web";

  //initial state
  private byte state = NOTAUTHORISED;
  
  /**
   * Main messages processing routing.
   */
  public String processInput(String data){
    String result = null;
    //initial status (processing of previous command)
    String status = STATUS_COMMAND_UNKNOWN;
    
    StringTokenizer st = new StringTokenizer(data);
    int tokensNumber = st.countTokens();
    String [] token = new String[tokensNumber];
    for(int i = 0; i < tokensNumber; i++)
      token[i] = st.nextToken();

    //command check
    if(commands.containsKey(data.toLowerCase())){
      byte command = commands.get(data.toLowerCase());
      switch(command){
        case COMMAND_HELP:
        {
          if(state == NOTAUTHORISED)
            result = getHelp(false);
          else
            result = getHelp(true);
          result = result.concat(STATUS_OK);
          break;
        }
        case COMMAND_LOGOUT:
        {
          if(state == AUTHORISED){
            state = NOTAUTHORISED;
            status = null;
          }
          break;
        }
        case COMMAND_EXIT:
        {
          if(state == NOTAUTHORISED)
            status = null;
          else
            result = "use 'logout' command instead\r\n".concat(STATUS_FAILED);
          break;
        }
        case COMMAND_STATUS:
        {
          if(state == AUTHORISED)
            result = "AUTHORISED\r\n".concat(STATUS_OK);
          else
            result = "NOTAUTHORISED\r\n".concat(STATUS_OK);
          break;
        }
        default:
        {
          status = STATUS_COMMAND_UNKNOWN;
        }
      }
    }else{
      if(commands.containsKey(token[0].toLowerCase())){
        byte command = commands.get(token[0].toLowerCase());
        switch(command){
          case COMMAND_LOGIN:
          {
            if(state == NOTAUTHORISED){
              if(token.length == 3 && LOGIN.equals(token[1]) && PASSWORD.equals(token[2])){
                state = AUTHORISED;
                status = STATUS_OK;
              }else{
                status = STATUS_FAILED;
              }
            }
            break;
          }
          case COMMAND_FILEINFO:
          {
            if(state == AUTHORISED){
              if(token.length == 2){
                File f = new File(token[1]);
                if(f.exists())
                  result = f.length()+"\r\n".concat(STATUS_OK);
                else
                  status = STATUS_FAILED;
              }else{
                status = STATUS_FAILED;
              }
            }
            break;
          }
          case COMMAND_FILEGET:
          {
            if(state == AUTHORISED){
              if(token.length == 2){
                File f = new File(token[1]);
                if(f.exists()){
                  BufferedInputStream in = null;
                  try{
                    int fileLength = (int)f.length();
                    byte [] outData = new byte[fileLength];
                    in = new BufferedInputStream(new FileInputStream(f));
                    int bytesRead = 0;
                    while(bytesRead < fileLength){
                      int read = in.read(outData, bytesRead, fileLength-bytesRead);
                      bytesRead += read;
                    }
                    in.close();
                    in = null;
                    result = new String(outData);//put Base64 enc here
                    status = STATUS_OK;
                  }catch(Exception ex){
                    ex.printStackTrace();
                    if(in != null)
                      try{ in.close(); in = null; }catch(Exception e){}
                    status = STATUS_FAILED;
                  }
                }else{
                  status = STATUS_FAILED;
                }
              }else{
                status = STATUS_FAILED;
              }
            }
            break;
          }
          default:
          {
            status = STATUS_COMMAND_UNKNOWN;
          }
        }
      }
    }
    
    return (result != null) ? result : status;
  }

  /**
   * Forms help message based on the user authorization state.
   */
  public String getHelp(boolean isAuthorized){
    String result = "Commands available:\r\n";
    if(isAuthorized){
      result = result.concat("  fileinfo [filename]\r\n");
      result = result.concat("    shows length of the specified file, if exists\r\n");
      result = result.concat("  fileget [filename]\r\n");
      result = result.concat("    returns content of the specified file, if exists\r\n");
      result = result.concat("  status\r\n");
      result = result.concat("    shows current status\r\n");
      result = result.concat("  logout\r\n");
      result = result.concat("    ends session\r\n");
    }else{
      result = result.concat("  login [name] [password]\r\n");
      result = result.concat("    logs you in\r\n");
      result = result.concat("  exit\r\n");
      result = result.concat("    ends session\r\n");
    }
    result = result.concat("  help\r\n");
    result = result.concat("    this message\r\n");
    return result;
  }

}
